@@ -1,27 +1,21 @@
-#!/usr/bin/perl
-
-use strict;
-use warnings;
-
-use Log::Log4perl qw(:easy);
-use YAML::Any qw(Dump);
+#!/usr/bin/env perl
 
 use FindBin;
 use lib "$FindBin::RealBin/../../lib";
 use lib "$FindBin::RealBin/../../../lib";
 
-use Utils qw(:all);
+use Test::Tutorial::Setup;
 
-init_logger($DEBUG);
+# init_logger($DEBUG);
 
 my $name = spreadsheet_name();
-my $sheets = sheets_api();
+my $sheets_api = sheets_api();
 # clean up any failed previous runs.
-$sheets->delete_all_spreadsheets($name);
-$sheets->rest_api()->post_process(\&show_api);
+$sheets_api->delete_all_spreadsheets($name);
+$sheets_api->rest_api()->api_callback(\&show_api);
 
 start("Now we will create a new spreadsheet named '$name'.");
-my $ss = $sheets->create_spreadsheet(title => $name);
+my $ss = $sheets_api->create_spreadsheet(title => $name);
 my $uri = $ss->spreadsheet_uri();
 end("Spreadsheet successfully created, uri: $uri.");
 
@@ -35,7 +29,7 @@ $ss_copy->delete_spreadsheet();
 end("Spreadsheet successfully deleted.");
 
 # clean up any failed previous runs.
-$sheets->delete_all_spreadsheets("${name}_copy");
+$sheets_api->delete_all_spreadsheets("${name}_copy");
 
 $uri = $ss->spreadsheet_uri();
 message('green', "\nOpen url '$uri' and proceed to the next step.\n");
