@@ -1,28 +1,42 @@
 package Test::Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range;
 
-use YAML::Any qw(Dump);
-use Test::Most;
+use Test::Unit::Setup;
 
-use Utils qw(:all);
-use Test::Mock::Worksheet;
+use parent 'Test::Unit::TestBase';
 
 use aliased 'Google::RestApi::SheetsApi4::Range';
-
-use parent qw(Test::Class Test::Google::RestApi::SheetsApi4::Base);
-
-sub class { 'Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range' }
+use aliased 'Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range' => 'Request::Range';
 
 my $index = {
-  sheetId          => 'mock_worksheet_id',
+  sheetId          => 'Sheet1',
   startColumnIndex => 0,
   startRowIndex    => 0,
   endColumnIndex   => 1,
   endRowIndex      => 1,
 };
 
+sub class { Request::Range; }
+
+sub setup : Tests(setup) {
+  my $self = shift;
+  $self->SUPER::setup(@_);
+
+  $self->_fake_http_auth();
+  $self->_fake_http_no_retries();
+
+  $self->_uri_responses(qw(
+    get_worksheet_properties_title_sheetid
+    post_worksheet_batch_request
+  ));
+
+  return;
+}
+
 sub range_text_format : Tests(29) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     repeatCell => {
       range => '',
@@ -35,7 +49,7 @@ sub range_text_format : Tests(29) {
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   $cell->{repeatCell}->{range} = $range->range_to_index();
 
   _range_text_format($cell, $range, 'bold');
@@ -113,6 +127,8 @@ sub _range_text_format_color {
 sub range_background_color : Tests(13) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     repeatCell => {
       range => '',
@@ -125,7 +141,7 @@ sub range_background_color : Tests(13) {
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   $cell->{repeatCell}->{range} = $range->range_to_index();
  
   _range_background_color($cell, $range, 'red', 1);
@@ -164,6 +180,8 @@ sub _range_background_color {
 sub range_misc : Tests(12) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     repeatCell => {
       range => '',
@@ -178,7 +196,7 @@ sub range_misc : Tests(12) {
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   $cell->{repeatCell}->{range} = $range->range_to_index();
   my $user = $cell->{repeatCell}->{cell}->{userEnteredFormat};
 
@@ -226,13 +244,15 @@ sub range_misc : Tests(12) {
 sub range_borders : Tests(22) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     updateBorders => {
       range => '',
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   my $borders = $cell->{updateBorders};
   $borders->{range} = $range->range_to_index();
 
@@ -282,6 +302,8 @@ sub _range_borders {
 sub range_border_style : Tests(14) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     updateBorders => {
       range => '',
@@ -291,7 +313,7 @@ sub range_border_style : Tests(14) {
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   $cell->{updateBorders}->{range} = $range->range_to_index();
 
   _range_border_style($cell, $range, $_)
@@ -318,6 +340,8 @@ sub _range_border_style {
 sub range_border_colors : Tests(13) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     updateBorders => {
       range => '',
@@ -327,7 +351,7 @@ sub range_border_colors : Tests(13) {
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   $cell->{updateBorders}->{range} = $range->range_to_index();
 
   _range_border_colors($cell, $range, 'red', 1);
@@ -364,6 +388,8 @@ sub _range_border_colors {
 sub range_border_cells : Tests(7) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     repeatCell => {
       range => '',
@@ -376,7 +402,7 @@ sub range_border_cells : Tests(7) {
     },
   };
 
-  my $range = $self->new_range("A1");
+  my $range = $self->_new_range("A1");
   $cell->{repeatCell}->{range} = $range->range_to_index();
   my $borders = $cell->{repeatCell}->{cell}->{userEnteredFormat}->{borders};
   my $fields = $cell->{repeatCell}->{fields};
@@ -400,6 +426,8 @@ sub range_border_cells : Tests(7) {
 sub range_merge : Tests(6) {
   my $self = shift;
 
+  $self->_fake_http_response_by_uri();
+
   my $cell = {
     mergeCells => {
       range     => '',
@@ -407,7 +435,7 @@ sub range_merge : Tests(6) {
     },
   };
 
-  my $range = $self->new_range("A1:B2");
+  my $range = $self->_new_range("A1:B2");
   $cell->{mergeCells}->{range} = $range->range_to_index();
   my @requests;
 
@@ -432,13 +460,11 @@ sub range_merge : Tests(6) {
   return;
 }
 
-sub new_range {
+sub _new_range {
   my $self = shift;
-  return Range->new(worksheet => $self->worksheet(), range => shift);
+  return Range->new(worksheet => fake_worksheet(), range => shift);
 }
 
-sub worksheet : Test(setup) { shift->{worksheet} = Test::Mock::Worksheet->new(); }
-
 sub _add_field {
   my ($cell, $field) = (@_);
   my %fields = map { $_ => 1; } split(',', $cell->{repeatCell}->{fields}), $field;
